CONTEXT_OPTIONS = {
    "ignore_https_errors": True,
    "color_scheme": "dark",
    "locale": "es-EC",
    "no_viewport": True,
    "timezone_id": "America/Guayaquil",
    "user_agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) "
    + "AppleWebKit/537.36 (KHTML, like Gecko) Chrome/139.0.0.0 "
    + "Safari/537.36 Edg/139.0.0.0",
    "extra_http_headers": {
        "accept-language": "es-EC",
        "sec-ch-ua": '"Chromium";v="139", "Not;A=Brand";v="99"',
        "sec-ch-ua-mobile": "?0",
        "sec-ch-ua-platform": '"Windows"',
    },
    "ignore_https_errors": True,
    "viewport": {"width": 1080, "height": 720},
}


STAPE_DOWNLOAD_URL = "https://streamtape.com/get_video"
YOURUPLOAD_DOWNLOAD_URL = "https://yourupload.com/embed"

YOURUPLOAD_TIMEOUT = 10000
SW_TIMEOUT = 7000
MEDIAFIRE_TIMEOUT = 10000
